#if 0
#define DPRINTK(fmt, args...)	printk(KERN_DEBUG "%s: " fmt, __FUNCTION__ , \
						  ## args)
#else
#define DPRINTK(fmt, args...)
#endif

#define p_crtc(arg) ((struct vesafb_crtc_ib*)(arg))
#define p_vbe(arg)  ((struct vesafb_vbe_ib*)(arg))
#define p_mode(arg) ((struct vesafb_mode_ib*)(arg))

struct vesafb_task {
	u8 flags;
	void *buf;
	int buf_len;
	struct vm86_regs regs;
	struct list_head node;
	struct completion done;
};

/* Vesafb task flags and masks */
#define TF_CALL		0x00
#define TF_EXIT		0x01
#define TF_GETVBEIB	0x02
#define TF_USE_BUF	0x04
#define TF_RETURN_BUF	0x08
#define TF_MASK_BUF	(TF_USE_BUF | TF_RETURN_BUF)

/* Macros and functions for manipulating vesafb tasks */
#define vesafb_create_task(task)				\
do { 								\
	task = kmalloc(sizeof(struct vesafb_task), GFP_ATOMIC); \
	if (task) 						\
		memset(task, 0, sizeof(struct vesafb_task));	\
	init_completion(&task->done);				\
} while (0)

#define vesafb_wait_for_task(task) 	wait_for_completion(&task->done);
#define vesafb_reset_task(task)		init_completion(&task->done);
int vesafb_queue_task(struct vesafb_task *task);

/* Functions for controlling the vesafb thread */
int vesafb_wait_for_thread(void);

#define VBE_CAP_CAN_SWITCH_DAC	0x01
#define VBE_CAP_VGACOMPAT	0x02

/* This struct is 512 bytes long */
struct vesafb_vbe_ib {
	char vbe_signature[4];
	u16  vbe_version;
	u32  oem_string_ptr;
	u32  capabilities;
	u32  mode_list_ptr;
	u16  total_memory;
	u16  oem_software_rev;
	u32  oem_vendor_name_ptr;
	u32  oem_product_name_ptr;
	u32  oem_product_rev_ptr;
	u8   reserved[222];
	char oem_data[256];
} __attribute__ ((packed));

struct vesafb_crtc_ib {
	u16 horiz_total;
	u16 horiz_start;
	u16 horiz_end;
	u16 vert_total;
	u16 vert_start;
	u16 vert_end;
	u8  flags;
	u32 pixel_clock;
	u16 refresh_rate;
	u8  reserved[40];
} __attribute__ ((packed));

#define VBE_MODE_VGACOMPAT	0x20

struct vesafb_mode_ib {
	/* for all VBE revisions */
	u16 mode_attr;
	u8  winA_attr;
	u8  winB_attr;
	u16 win_granularity;
	u16 win_size;
	u16 winA_seg;
	u16 winB_seg;
	u32 win_func_ptr;
	u16 bytes_per_scan_line;

	/* for VBE 1.2+ */
	u16 x_res;
	u16 y_res;
	u8  x_char_size;
	u8  y_char_size;
	u8  planes;
	u8  bits_per_pixel;
	u8  banks;
	u8  memory_model;
	u8  bank_size;
	u8  image_pages;
	u8  reserved1;

	/* Direct color fields for direct/6 and YUV/7 memory models. */
	/* Offsets are bit positions of lsb in the mask. */
	u8  red_len;
	u8  red_off;
	u8  green_len;
	u8  green_off;
	u8  blue_len;
	u8  blue_off;
	u8  rsvd_len;
	u8  rsvd_off;
	u8  direct_color_info;	/* direct color mode attributes */

	/* for VBE 2.0+ */
	u32 phys_base_ptr;
	u8  reserved2[6];

	/* for VBE 3.0+ */
	u16 lin_bytes_per_scan_line;
	u8  bnk_image_pages;
	u8  lin_image_pages;
	u8  lin_red_len;
	u8  lin_red_off;
	u8  lin_green_len;
	u8  lin_green_off;
	u8  lin_blue_len;
	u8  lin_blue_off;
	u8  lin_rsvd_len;
	u8  lin_rsvd_off;
	u32 max_pixel_clock;
	u16 mode_id;
	u8  depth;
} __attribute__ ((packed));

struct vesafb_pal_entry {
	u_char blue, green, red, pad;
} __attribute__ ((packed));

struct vesafb_par {
	u32 mem_total;
	int mode_idx;
	struct vesafb_crtc_ib crtc;
};

